# Generated by roxygen2: do not edit by hand

S3method("[",lobstr_bytes)
S3method(c,lobstr_bytes)
S3method(format,lobstr_bytes)
S3method(format,lobstr_inspector)
S3method(print,lobstr_bytes)
S3method(print,lobstr_inspector)
S3method(print,lobstr_raw)
S3method(print,lobstr_srcref)
S3method(tree_label,"NULL")
S3method(tree_label,"function")
S3method(tree_label,character)
S3method(tree_label,default)
S3method(tree_label,environment)
S3method(tree_label,lobstr_srcfile_ref)
S3method(tree_label,lobstr_srcref)
S3method(tree_label,lobstr_srcref_location)
S3method(tree_label,srcfile)
S3method(tree_label,srcref)
export(ast)
export(cst)
export(mem_used)
export(obj_addr)
export(obj_addrs)
export(obj_size)
export(obj_sizes)
export(ref)
export(src)
export(sxp)
export(tree)
export(tree_label)
import(rlang)
useDynLib(lobstr, .registration = TRUE)
