# Generated by cpp11: do not edit by hand

obj_addr_ <- function(name, env) {
  .Call(`_lobstr_obj_addr_`, name, env)
}

obj_addrs_ <- function(x) {
  .Call(`_lobstr_obj_addrs_`, x)
}

obj_inspect_ <- function(x, max_depth, expand_char, expand_altrep, expand_env, expand_call, expand_bytecode) {
  .Call(`_lobstr_obj_inspect_`, x, max_depth, expand_char, expand_altrep, expand_env, expand_call, expand_bytecode)
}

init_library <- function(env) {
  invisible(.Call(`_lobstr_init_library`, env))
}

v_size <- function(n, element_size) {
  .Call(`_lobstr_v_size`, n, element_size)
}

obj_size_ <- function(objects, base_env, sizeof_node, sizeof_vector) {
  .Call(`_lobstr_obj_size_`, objects, base_env, sizeof_node, sizeof_vector)
}

obj_csize_ <- function(objects, base_env, sizeof_node, sizeof_vector) {
  .Call(`_lobstr_obj_csize_`, objects, base_env, sizeof_node, sizeof_vector)
}
