% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lobstr-package.R
\docType{package}
\name{lobstr-package}
\alias{lobstr}
\alias{lobstr-package}
\title{lobstr: Visualize R Data Structures with Trees}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

A set of tools for inspecting and understanding R data structures inspired by str(). Includes ast() for visualizing abstract syntax trees, ref() for showing shared references, cst() for showing call stack trees, and obj_size() for computing object sizes.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://lobstr.r-lib.org/}
  \item \url{https://github.com/r-lib/lobstr}
  \item Report bugs at \url{https://github.com/r-lib/lobstr/issues}
}

}
\author{
\strong{Maintainer}: Hadley Wickham \email{hadley@posit.co}

Authors:
\itemize{
  \item Hadley Wickham \email{hadley@posit.co}
}

Other contributors:
\itemize{
  \item Posit Software, PBC [copyright holder, funder]
}

}
\keyword{internal}
